



package name.panitz.rmi;

import java.rmi.*;
import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;

import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import static java.awt.event.KeyEvent.*;

public interface RMI{




  interface GetDate extends Remote {
    Date getDate() throws RemoteException;
}


  class GetDateImpl implements GetDate {
    public Date getDate() {return new Date();}
  }


  static class DateServer {
    public static void main(String args[]) throws Exception{
      var obj = new GetDateImpl();
      var stub = (GetDate) UnicastRemoteObject.exportObject(obj, 0);
      LocateRegistry.getRegistry().bind("date", stub);
      System.err.println("Server ready");
    }
  }


  static class DateClient extends JPanel {
    JButton getButton = new JButton("get new time from server");
    JLabel l    = new JLabel();
    DateClient(String host){
      add(getButton);
      add(l);
      try {
        var registry = LocateRegistry.getRegistry(host);
        var stub = (GetDate) registry.lookup("date");
        l.setText(""+stub.getDate());

        getButton.addActionListener(e->{
          try {l.setText(""+stub.getDate());}catch (Exception e1){}
        });
      }catch (Exception e) {}
    }

    public static void main(String[] args) {
      JFrame f = new JFrame();
      f.add(new DateClient(args[0]));
      f.pack();
      f.setVisible(true);
    }
  }


  static interface BallGame extends Remote {
    void createBall(String name) throws RemoteException;
    void move(String name,int x,int y) throws RemoteException;
    Map<String,Dimension> getBalls()throws RemoteException;
}


  static  class BallGameImpl implements BallGame {
    private Map<String,Dimension> bs
      = new HashMap<String,Dimension>();

    public void createBall(String name){
      bs.put(name,new Dimension(10,10));}

    public void move(String name,int x,int y){
      Dimension d=bs.get(name);
      if (d!=null) d.setSize(d.width+x,d.height+y);
    }

    public Map<String,Dimension> getBalls(){return bs;}
}


  static  class BallGameServer {
    public static void main(String args[]) {
      try {
        var obj = new BallGameImpl();
        var stub = (BallGame) UnicastRemoteObject.exportObject(obj, 0);
        LocateRegistry.getRegistry().bind("ballgame", stub);
        System.err.println("Server ready");
      } catch (Exception e) {
        System.err.println("Server exception: " + e.toString());
        e.printStackTrace();
      }
    }
  }


  static interface Animation {
    public void move();
  }


  static abstract class AnimatedJPanel extends JPanel implements Animation{



  javax.swing.Timer t;


    public AnimatedJPanel(){
      super(true);
      t = new javax.swing.Timer(29,ev->{
          move();
          repaint();
        });
      t.start();
    }
  }


  static class BallGameClient extends AnimatedJPanel {
    final String name;
    BallGame game=null;
    final int width = 400;
    final int height = 300;



    BallGameClient(String host,String n){
      this.name=n;
      setFocusable(true);
      try {
        Registry registry = LocateRegistry.getRegistry(host);
        game = (BallGame) registry.lookup("ballgame");
      } catch (Exception e) {}




      addMouseListener(new MouseAdapter(){
        public void mouseClicked(MouseEvent e){
          try{game.createBall(name);}catch(Exception e1){}
        }
      });



      addKeyListener(new KeyAdapter(){
        public void keyReleased(KeyEvent e){
          try{
            switch (e.getKeyCode()){
              case VK_DOWN: game.move(name,0,2);break;  
              case VK_LEFT:game.move(name,-2,0);break;
              case VK_RIGHT:game.move(name,2,0);break;
              case VK_UP:game.move(name,0,-2);break;
            }
          }catch(Exception e1){System.out.println(e);}
        }
      });
    }



    public void paintComponent(Graphics g){
      g.setColor(Color.WHITE);
      g.fillRect(0,0,width,height);
      try{
        for (Dimension d:game.getBalls().values()){
          g.setColor(Color.RED);
          g.fillOval(d.height,d.width,10,10);
        }
      }catch (Exception e){}
    }



    public Dimension getPreferredSize(){
      return new Dimension(width,height);}



    public static void main(String[] args) {
      JFrame f = new JFrame();
      f.add(new BallGameClient(args[0],args[1]));
      f.pack();
      f.setVisible(true);
    }
    public void move(){}
  }

}